@@ -784,12 +784,13 @@ int main(int argc, char **argv) {
 
     apps_pids_init();
     OS_FUNCTION(apps_os_init)();
+    int exit_status = 0;
 
     // ------------------------------------------------------------------------
     // the event loop for functions
 
     struct functions_evloop_globals *wg =
-            functions_evloop_init(1, "APPS", &apps_and_stdout_mutex, &apps_plugin_exit);
+        functions_evloop_init(1, "APPS", &apps_and_stdout_mutex, &apps_plugin_exit, &exit_status);
 
     functions_evloop_add_function(wg, "processes", function_processes, PLUGINS_FUNCTIONS_TIMEOUT_DEFAULT, NULL);
 
@@ -801,7 +802,7 @@ int main(int argc, char **argv) {
     global_iterations_counter = 1;
     heartbeat_t hb;
     heartbeat_init(&hb, update_every * USEC_PER_SEC);
-    for(; !apps_plugin_exit ; global_iterations_counter++) {
+    for (; !__atomic_load_n(&apps_plugin_exit, __ATOMIC_ACQUIRE); global_iterations_counter++) {
         netdata_mutex_unlock(&apps_and_stdout_mutex);
 
         usec_t dt;
@@ -881,4 +882,5 @@ int main(int argc, char **argv) {
         debug_log("done Loop No %zu", global_iterations_counter);
     }
     netdata_mutex_unlock(&apps_and_stdout_mutex);
+    exit(exit_status);
 }
