@@ -219,6 +219,7 @@ int progress_unittest(void);
 int dyncfg_unittest(void);
 int eval_unittest(void);
 int duration_unittest(void);
+int health_config_unittest(void);
 bool netdata_random_session_id_generate(void);
 
 #ifdef OS_WINDOWS
@@ -407,6 +408,7 @@ int netdata_main(int argc, char **argv) {
                             if (dyncfg_unittest()) return 1;
                             if (eval_unittest()) return 1;
                             if (duration_unittest()) return 1;
+                            if (health_config_unittest()) return 1;
                             if (unittest_waiting_queue()) return 1;
                             if (uuidmap_unittest()) return 1;
 #ifdef HAVE_LIBBACKTRACE
@@ -527,6 +529,10 @@ int netdata_main(int argc, char **argv) {
                             unittest_running = true;
                             return duration_unittest();
                         }
+                        else if(strcmp(optarg, "healthconfigtest") == 0) {
+                            unittest_running = true;
+                            return health_config_unittest();
+                        }
                         else if(strcmp(optarg, "dyncfgtest") == 0) {
                             unittest_running = true;
                             if(unittest_prepare_rrd(&user))
