@@ -100,7 +100,7 @@ static void daemon_status_file_to_json(BUFFER *wb, DAEMON_STATUS_FILE *ds) {
 
     dsf_acquire(*ds);
 
-    buffer_json_member_add_datetime_rfc3339(wb, "@timestamp", ds->timestamp_ut, true);
+    buffer_json_member_add_string(wb, "@timestamp", ds->timestamp_ut_rfc3339);
     buffer_json_member_add_uint64(wb, "version", STATUS_FILE_VERSION);
 
     buffer_json_member_add_object(wb, "agent");
@@ -108,7 +108,7 @@ static void daemon_status_file_to_json(BUFFER *wb, DAEMON_STATUS_FILE *ds) {
         buffer_json_member_add_uuid(wb, "id", ds->host_id.uuid.uuid);
 
         if(ds->v >= 24 && ds->host_id.last_modified_ut)
-            buffer_json_member_add_datetime_rfc3339(wb, "since", ds->host_id.last_modified_ut, true);
+            buffer_json_member_add_string(wb, "since", ds->host_id.last_modified_ut_rfc3339);
 
         buffer_json_member_add_uuid_compact(wb, "ephemeral_id", ds->invocation.uuid);
         buffer_json_member_add_string(wb, "version", ds->version);
@@ -246,7 +246,7 @@ static void daemon_status_file_to_json(BUFFER *wb, DAEMON_STATUS_FILE *ds) {
             buffer_json_member_add_uint64(wb, "dbengine", ds->disk_footprint.dbengine);
             buffer_json_member_add_uint64(wb, "sqlite", ds->disk_footprint.sqlite);
             buffer_json_member_add_uint64(wb, "other", ds->disk_footprint.other);
-            buffer_json_member_add_datetime_rfc3339(wb, "last_updated", ds->disk_footprint.last_updated_ut, true);
+            buffer_json_member_add_string(wb, "last_updated", ds->disk_footprint.last_updated_ut_rfc3339);
             buffer_json_object_close(wb);
         }
         buffer_json_object_close(wb);
@@ -748,6 +748,8 @@ static void daemon_status_file_refresh(DAEMON_STATUS status) {
     session_status.boottime = now_boottime_sec();
     session_status.uptime = now_realtime_sec() - netdata_start_time;
     session_status.timestamp_ut = now_ut;
+    rfc3339_datetime_ut(session_status.timestamp_ut_rfc3339, sizeof(session_status.timestamp_ut_rfc3339),
+                        session_status.timestamp_ut, 2, true);
     session_status.invocation = nd_log_get_invocation_id();
     session_status.db_mode = default_rrd_memory_mode;
     session_status.db_tiers = nd_profile.storage_tiers;
@@ -829,10 +831,13 @@ static void daemon_status_file_refresh(DAEMON_STATUS status) {
         
         // Calculate other files (total - dbengine - sqlite)
         session_status.disk_footprint.other = total_size.bytes - dbengine_size.bytes - sqlite_size.bytes;
-        
+
         // Update last updated timestamp
         session_status.disk_footprint.last_updated_ut = now_ut;
-        
+        rfc3339_datetime_ut(session_status.disk_footprint.last_updated_ut_rfc3339,
+                            sizeof(session_status.disk_footprint.last_updated_ut_rfc3339),
+                            session_status.disk_footprint.last_updated_ut, 2, true);
+
         // Clean up patterns
         simple_pattern_free(dbengine_pattern);
         simple_pattern_free(sqlite_pattern);
@@ -919,6 +924,14 @@ static const char *agent_health(DAEMON_STATUS_FILE *ds) {
         return "healthy-recovered";
 }
 
+// Callback to discard curl response data.
+// Without this, curl writes to stdout which can crash on Windows when running as a service.
+static size_t post_status_file_discard_response(void *ptr, size_t size, size_t nmemb, void *userdata) {
+    (void)ptr;
+    (void)userdata;
+    return size * nmemb;
+}
+
 static void post_status_file(struct post_status_file_thread_data *d) {
     daemon_status_file_startup_step("startup(crash reports json)");
 
@@ -948,6 +961,15 @@ static void post_status_file(struct post_status_file_thread_data *d) {
     curl_easy_setopt(curl, CURLOPT_URL, "https://agent-events.netdata.cloud/agent-events");
     curl_easy_setopt(curl, CURLOPT_POST, 1L);
     curl_easy_setopt(curl, CURLOPT_POSTFIELDS, json_data);
+
+    // Discard response data - without this curl writes to stdout which can crash
+    // on Windows when running as a service or without a console
+    curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, post_status_file_discard_response);
+    curl_easy_setopt(curl, CURLOPT_WRITEDATA, NULL);
+
+    // Prevent signal-based timeouts which can cause issues on Windows/MSYS2
+    curl_easy_setopt(curl, CURLOPT_NOSIGNAL, 1L);
+
     struct curl_slist *headers = NULL;
     headers = curl_slist_append(headers, "Content-Type: application/json");
     curl_easy_setopt(curl, CURLOPT_HTTPHEADER, headers);
@@ -1062,10 +1084,27 @@ void daemon_status_file_init(void) {
 
     if(last_session_status.v <= 26)
         fill_dmi_info(&last_session_status);
-    
+
     if(last_session_status.v <= 27)
         last_session_status.system_cpus = os_get_system_cpus();
 
+    // Regenerate RFC3339 strings from loaded timestamps for async-signal-safe compatibility
+    // (these fields don't exist in saved JSON, they're runtime-only for signal handlers)
+    if(last_session_status.timestamp_ut)
+        rfc3339_datetime_ut(last_session_status.timestamp_ut_rfc3339,
+                            sizeof(last_session_status.timestamp_ut_rfc3339),
+                            last_session_status.timestamp_ut, 2, true);
+
+    if(last_session_status.host_id.last_modified_ut)
+        rfc3339_datetime_ut(last_session_status.host_id.last_modified_ut_rfc3339,
+                            sizeof(last_session_status.host_id.last_modified_ut_rfc3339),
+                            last_session_status.host_id.last_modified_ut, 2, true);
+
+    if(last_session_status.disk_footprint.last_updated_ut)
+        rfc3339_datetime_ut(last_session_status.disk_footprint.last_updated_ut_rfc3339,
+                            sizeof(last_session_status.disk_footprint.last_updated_ut_rfc3339),
+                            last_session_status.disk_footprint.last_updated_ut, 2, true);
+
     daemon_status_file_migrate_once();
 }
 
@@ -1540,8 +1579,8 @@ void daemon_status_file_shutdown_timeout(BUFFER *trace) {
 
     safecpy(session_status.fatal.function, "shutdown_timeout");
 
-    CLEAN_BUFFER *wb = buffer_create(0, NULL);
-    daemon_status_file_save(wb, &session_status, false);
+    static_save_buffer_init();
+    daemon_status_file_save(static_save_buffer, &session_status, false);
 
     // keep the spinlock locked, to prevent further steps updating the status
 }
