@@ -337,6 +337,9 @@ done:
 
 void sql_health_alarm_log_save(RRDHOST *host, ALARM_ENTRY *ae)
 {
+    if (!REQUIRE_HEALTH_DB_OPEN())
+        return;
+
     if (ae->flags & HEALTH_ENTRY_FLAG_SAVED)
         sql_health_alarm_log_update(host, ae);
     else
@@ -555,6 +558,8 @@ void sql_check_removed_alerts_state(RRDHOST *host)
 
            sql_inject_removed_status(host, alarm_id, alarm_event_id, unique_id, ++max_unique_id, &transition_id);
         }
+        if (!service_running(SERVICE_HEALTH))
+            break;
     }
 done:
     REPORT_BIND_FAIL(res, param);
@@ -943,6 +948,9 @@ done:
 
 int sql_health_get_last_executed_event(RRDHOST *host, ALARM_ENTRY *ae, RRDCALC_STATUS *last_executed_status)
 {
+    if (!REQUIRE_HEALTH_DB_OPEN())
+        return -1;
+
     int ret = -1;
     static __thread sqlite3_stmt *compiled_res = NULL;
     sqlite3_stmt *res = NULL;
@@ -1111,7 +1119,7 @@ void sql_health_alarm_log2json(RRDHOST *host, BUFFER *wb, time_t after, const ch
         if (sqlite3_column_type(stmt_query, 24) == SQLITE_NULL)
             buffer_json_member_add_string(wb, "old_value", NULL);
         else
-            buffer_json_member_add_double(wb, "old_value", sqlite3_column_double(stmt_query, 23));
+            buffer_json_member_add_double(wb, "old_value", sqlite3_column_double(stmt_query, 24));
 
         freez(edit_command);
 
