@@ -858,3 +858,41 @@ int ssl_security_location_for_context(SSL_CTX *ctx, const char *file, const char
 
     return 0;
 }
+
+void netdata_ssl_log_verify_error(X509_STORE_CTX *ctx) {
+    int err   = X509_STORE_CTX_get_error(ctx);
+    int depth = X509_STORE_CTX_get_error_depth(ctx);
+
+    BIO *bio = NULL;
+    const char *subject = NULL;
+    int subject_len = 0;
+
+    X509 *cert = X509_STORE_CTX_get_current_cert(ctx);
+    if (cert) {
+        X509_NAME *name = X509_get_subject_name(cert);
+        if (name) {
+            bio = BIO_new(BIO_s_mem());
+            if (bio && X509_NAME_print_ex(bio, name, 0, XN_FLAG_ONELINE) >= 0) {
+                BUF_MEM *bptr = NULL;
+                /* Optional defensive check:
+                 * BIO_get_mem_ptr() returns 1 on success, 0 on failure. */
+                if (BIO_get_mem_ptr(bio, &bptr) > 0 && bptr && bptr->data && bptr->length > 0) {
+                    subject = bptr->data;
+                    subject_len = (int)bptr->length;
+                }
+            }
+        }
+    }
+
+    if (subject && subject_len > 0) {
+        nd_log(NDLS_DAEMON, NDLP_ERR,
+               "SSL: certificate verify error %d:%s at depth %d, subject: %.*s",
+               err, X509_verify_cert_error_string(err), depth, subject_len, subject);
+    } else {
+        nd_log(NDLS_DAEMON, NDLP_ERR,
+               "SSL: certificate verify error %d:%s at depth %d",
+               err, X509_verify_cert_error_string(err), depth);
+    }
+
+    BIO_free(bio);
+}
