@@ -56,13 +56,28 @@ ml_dimension_calculated_numbers(ml_worker_t *worker, ml_dimension_t *dim)
     training_response.first_entry_on_response = rrddim_first_entry_s_of_tier(dim->rd, 0);
     training_response.last_entry_on_response = rrddim_last_entry_s_of_tier(dim->rd, 0);
 
-    size_t min_n = Cfg.min_train_samples;
-    size_t max_n = Cfg.max_train_samples;
+    unsigned chart_update_every = dim->rd->rrdset->update_every;
+    size_t smoothing_window = (chart_update_every > nd_profile.update_every) ? 1 : Cfg.max_samples_to_smooth;
+    size_t min_required_samples = Cfg.diff_n + smoothing_window + Cfg.lag_n;
+
+    auto round_up_div = [](time_t window, unsigned step) -> size_t {
+        if (window <= 0 || step == 0)
+            return 0;
+        return static_cast<size_t>((window + step - 1) / step);
+    };
+
+    size_t min_n = round_up_div(Cfg.min_training_window, chart_update_every);
+    size_t max_n = round_up_div(Cfg.training_window, chart_update_every);
+
+    if (min_n < min_required_samples)
+        min_n = min_required_samples;
+    if (max_n < min_required_samples)
+        max_n = min_required_samples;
 
     // Figure out what our time window should be.
     training_response.query_before_t = training_response.last_entry_on_response;
     training_response.query_after_t = std::max(
-        training_response.query_before_t - static_cast<time_t>((max_n - 1) * dim->rd->rrdset->update_every),
+        training_response.query_before_t - Cfg.training_window,  // Fixed time window
         training_response.first_entry_on_response
     );
 
@@ -125,6 +140,9 @@ ml_dimension_calculated_numbers(ml_worker_t *worker, ml_dimension_t *dim)
     if (idx != 0)
         memmove(worker->training_cns, &worker->training_cns[idx], sizeof(calculated_number_t) * training_response.total_values);
 
+    if (training_response.total_values < min_required_samples)
+        return { ML_WORKER_RESULT_NOT_ENOUGH_COLLECTED_VALUES, training_response };
+
     return { ML_WORKER_RESULT_OK, training_response };
 }
 
@@ -150,7 +168,9 @@ const char *db_models_add_model =
     "    @c10, @c11, @c12, @c13, @c14, @c15);";
 
 const char *db_models_load =
-    "SELECT * FROM models "
+    "SELECT after, before, min_dist, max_dist, "
+    "c00, c01, c02, c03, c04, c05, "
+    "c10, c11, c12, c13, c14, c15 FROM models "
     "WHERE dim_id = @dim_id AND after >= @after ORDER BY before ASC;";
 
 const char *db_models_delete =
@@ -379,7 +399,7 @@ int ml_dimension_load_models(RRDDIM *rd, sqlite3_stmt **active_stmt) {
     if (unlikely(rc != SQLITE_OK))
         goto bind_fail;
 
-    rc = sqlite3_bind_int64(res, ++param, now_realtime_sec() - (Cfg.num_models_to_use * Cfg.max_train_samples));
+    rc = sqlite3_bind_int64(res, ++param, now_realtime_sec() - (Cfg.num_models_to_use * Cfg.train_every));
     if (unlikely(rc != SQLITE_OK))
         goto bind_fail;
 
@@ -389,29 +409,29 @@ int ml_dimension_load_models(RRDDIM *rd, sqlite3_stmt **active_stmt) {
     while ((rc = sqlite3_step_monitored(res)) == SQLITE_ROW) {
         ml_kmeans_t km;
 
-        km.after = sqlite3_column_int(res, 2);
-        km.before = sqlite3_column_int(res, 3);
+        km.after = sqlite3_column_int(res, 0);
+        km.before = sqlite3_column_int(res, 1);
 
-        km.min_dist = sqlite3_column_int(res, 4);
-        km.max_dist = sqlite3_column_int(res, 5);
+        km.min_dist = sqlite3_column_double(res, 2);
+        km.max_dist = sqlite3_column_double(res, 3);
 
         km.cluster_centers.resize(2);
 
         km.cluster_centers[0].set_size(Cfg.lag_n + 1);
-        km.cluster_centers[0](0) = sqlite3_column_double(res, 6);
-        km.cluster_centers[0](1) = sqlite3_column_double(res, 7);
-        km.cluster_centers[0](2) = sqlite3_column_double(res, 8);
-        km.cluster_centers[0](3) = sqlite3_column_double(res, 9);
-        km.cluster_centers[0](4) = sqlite3_column_double(res, 10);
-        km.cluster_centers[0](5) = sqlite3_column_double(res, 11);
+        km.cluster_centers[0](0) = sqlite3_column_double(res, 4);
+        km.cluster_centers[0](1) = sqlite3_column_double(res, 5);
+        km.cluster_centers[0](2) = sqlite3_column_double(res, 6);
+        km.cluster_centers[0](3) = sqlite3_column_double(res, 7);
+        km.cluster_centers[0](4) = sqlite3_column_double(res, 8);
+        km.cluster_centers[0](5) = sqlite3_column_double(res, 9);
 
         km.cluster_centers[1].set_size(Cfg.lag_n + 1);
-        km.cluster_centers[1](0) = sqlite3_column_double(res, 12);
-        km.cluster_centers[1](1) = sqlite3_column_double(res, 13);
-        km.cluster_centers[1](2) = sqlite3_column_double(res, 14);
-        km.cluster_centers[1](3) = sqlite3_column_double(res, 15);
-        km.cluster_centers[1](4) = sqlite3_column_double(res, 16);
-        km.cluster_centers[1](5) = sqlite3_column_double(res, 17);
+        km.cluster_centers[1](0) = sqlite3_column_double(res, 10);
+        km.cluster_centers[1](1) = sqlite3_column_double(res, 11);
+        km.cluster_centers[1](2) = sqlite3_column_double(res, 12);
+        km.cluster_centers[1](3) = sqlite3_column_double(res, 13);
+        km.cluster_centers[1](4) = sqlite3_column_double(res, 14);
+        km.cluster_centers[1](5) = sqlite3_column_double(res, 15);
 
         dim->km_contexts.emplace_back(km);
     }
@@ -567,7 +587,7 @@ ml_dimension_deserialize_kmeans(const char *json_str)
     return true;
 }
 
-static void ml_dimension_stream_kmeans(const ml_dimension_t *dim)
+static void ml_dimension_stream_kmeans(ml_worker_t *worker, const ml_dimension_t *dim)
 {
     struct sender_state *s = dim->rd->rrdset->rrdhost->sender;
     if (!s)
@@ -578,10 +598,13 @@ static void ml_dimension_stream_kmeans(const ml_dimension_t *dim)
         !rrddim_check_upstream_exposed(dim->rd))
         return;
 
-    CLEAN_BUFFER *payload = buffer_create(0, NULL);
+    // Reuse worker's buffers instead of allocating new ones
+    BUFFER *payload = worker->stream_payload_buffer;
+    buffer_flush(payload);
     ml_dimension_serialize_kmeans(dim, payload);
 
-    CLEAN_BUFFER *wb = buffer_create(0, NULL);
+    BUFFER *wb = worker->stream_wb_buffer;
+    buffer_flush(wb);
 
     buffer_sprintf(
         wb, PLUGINSD_KEYWORD_JSON " " PLUGINSD_KEYWORD_JSON_CMD_ML_MODEL "\n%s\n" PLUGINSD_KEYWORD_JSON_END "\n",
@@ -632,7 +655,7 @@ static void ml_dimension_update_models(ml_worker_t *worker, ml_dimension_t *dim)
     model_info.inlined_kmeans = dim->km_contexts.back();
     worker->pending_model_info.push_back(model_info);
 
-    ml_dimension_stream_kmeans(dim);
+    ml_dimension_stream_kmeans(worker, dim);
 
     // Clear the training in progress flag
     dim->training_in_progress = false;
@@ -685,13 +708,29 @@ ml_dimension_train_model(ml_worker_t *worker, ml_dimension_t *dim)
         memcpy(worker->scratch_training_cns, worker->training_cns,
                training_response.total_values * sizeof(calculated_number_t));
 
+        size_t smoothing_window = (dim->rd->rrdset->update_every > nd_profile.update_every) ? 1 : Cfg.max_samples_to_smooth;
+
         ml_features_t features = {
-            Cfg.diff_n, Cfg.smooth_n, Cfg.lag_n,
+            Cfg.diff_n, smoothing_window, Cfg.lag_n,
             worker->scratch_training_cns, training_response.total_values,
             worker->training_cns, training_response.total_values,
             worker->training_samples
         };
-        ml_features_preprocess(&features);
+        
+        // Calculate dynamic sampling ratio based on expected output size
+        // After diff and smooth, we'll have approximately this many vectors
+        size_t expected_vectors = training_response.total_values;
+        if (Cfg.diff_n > 0) expected_vectors--;
+        if (smoothing_window > 1) expected_vectors = expected_vectors - smoothing_window + 1;
+        expected_vectors = expected_vectors - Cfg.lag_n;
+        
+        double sampling_ratio = 1.0;
+        if (expected_vectors > Cfg.max_training_vectors) {
+            sampling_ratio = (double)Cfg.max_training_vectors / expected_vectors;
+        }
+
+        // Apply sampling during lag feature extraction
+        ml_features_preprocess(&features, sampling_ratio);
 
         ml_kmeans_init(&dim->kmeans);
         ml_kmeans_train(&dim->kmeans, &features,  Cfg.max_kmeans_iters, training_response.query_after_t, training_response.query_before_t);
@@ -710,16 +749,22 @@ ml_dimension_predict(ml_dimension_t *dim, calculated_number_t value, bool exists
     if (dim->mls != MACHINE_LEARNING_STATUS_ENABLED)
         return false;
 
+    // Acquire lock to protect dim->cns from concurrent access by ml_host_stop()
+    if (spinlock_trylock(&dim->slock) == 0)
+        return false;
+
     // Don't treat values that don't exist as anomalous
     if (!exists) {
         dim->cns.clear();
+        spinlock_unlock(&dim->slock);
         return false;
     }
 
     // Save the value and return if we don't have enough values for a sample
-    unsigned n = Cfg.diff_n + Cfg.smooth_n + Cfg.lag_n;
+    unsigned n = Cfg.diff_n + Cfg.max_samples_to_smooth + Cfg.lag_n;
     if (dim->cns.size() < n) {
         dim->cns.push_back(value);
+        spinlock_unlock(&dim->slock);
         return false;
     }
 
@@ -742,17 +787,11 @@ ml_dimension_predict(ml_dimension_t *dim, calculated_number_t value, bool exists
     memcpy(dst_cns, dim->cns.data(), n * sizeof(calculated_number_t));
 
     ml_features_t features = {
-        Cfg.diff_n, Cfg.smooth_n, Cfg.lag_n,
+        Cfg.diff_n, Cfg.max_samples_to_smooth, Cfg.lag_n,
         dst_cns, n, src_cns, n,
         dim->feature
     };
-    ml_features_preprocess(&features);
-
-    /*
-     * Lock to predict
-    */
-    if (spinlock_trylock(&dim->slock) == 0)
-        return false;
+    ml_features_preprocess(&features, 1.0);
 
     // Mark the metric time as variable if we received different values
     if (!same_value)
@@ -1005,6 +1044,7 @@ void ml_detect_main(void *arg)
 }
 
 static void ml_flush_pending_models(ml_worker_t *worker) {
+    static time_t next_vacuum_run = 0;
     int op_no = 1;
 
     // begin transaction
@@ -1052,7 +1092,7 @@ static void ml_flush_pending_models(ml_worker_t *worker) {
         worker->num_models_to_prune += worker->pending_model_info.size();
     }
 
-    vacuum_database(ml_db, "ML", 0, 0);
+    vacuum_database(ml_db, "ML", 0, 0, &next_vacuum_run);
 
     worker->pending_model_info.clear();
 }
